@@ -4,7 +4,7 @@ import os
 import sys
 import tempfile
 import unittest
-from collections import namedtuple
+import warnings
 from io import StringIO, BytesIO
 
 class HackedSysModule:
@@ -119,27 +119,14 @@ def gen_result(data, environ):
 
 class CgiTests(unittest.TestCase):
 
-    def test_parse_multipart(self):
-        fp = BytesIO(POSTDATA.encode('latin1'))
-        env = {'boundary': BOUNDARY.encode('latin1'),
-               'CONTENT-LENGTH': '558'}
-        result = cgi.parse_multipart(fp, env)
-        expected = {'submit': [b' Add '], 'id': [b'1234'],
-                    'file': [b'Testing 123.\n'], 'title': [b'']}
-        self.assertEqual(result, expected)
-
-    def test_fieldstorage_properties(self):
-        fs = cgi.FieldStorage()
-        self.assertFalse(fs)
-        self.assertIn("FieldStorage", repr(fs))
-        self.assertEqual(list(fs), list(fs.keys()))
-        fs.list.append(namedtuple('MockFieldStorage', 'name')('fieldvalue'))
-        self.assertTrue(fs)
-
     def test_escape(self):
-        self.assertEqual("test &amp; string", cgi.escape("test & string"))
-        self.assertEqual("&lt;test string&gt;", cgi.escape("<test string>"))
-        self.assertEqual("&quot;test string&quot;", cgi.escape('"test string"', True))
+        # cgi.escape() is deprecated.
+        with warnings.catch_warnings():
+            warnings.filterwarnings('ignore', 'cgi\.escape',
+                                     DeprecationWarning)
+            self.assertEqual("test &amp; string", cgi.escape("test & string"))
+            self.assertEqual("&lt;test string&gt;", cgi.escape("<test string>"))
+            self.assertEqual("&quot;test string&quot;", cgi.escape('"test string"', True))
 
     def test_strict(self):
         for orig, expect in parse_strict_test_cases:
@@ -169,8 +156,7 @@ class CgiTests(unittest.TestCase):
 
     def test_log(self):
         cgi.log("Testing")
-        cgi.logfile = "fail/"
-        cgi.initlog("%s", "Testing initlog")
+
         cgi.logfp = StringIO()
         cgi.initlog("%s", "Testing initlog 1")
         cgi.log("%s", "Testing log 2")
@@ -179,13 +165,7 @@ class CgiTests(unittest.TestCase):
             cgi.logfp = None
             cgi.logfile = "/dev/null"
             cgi.initlog("%s", "Testing log 3")
-            def log_cleanup():
-                """Restore the global state of the log vars."""
-                cgi.logfile = ''
-                cgi.logfp.close()
-                cgi.logfp = None
-                cgi.log = cgi.initlog
-            self.addCleanup(log_cleanup)
+            self.addCleanup(cgi.closelog)
             cgi.log("Testing log 4")
 
     def test_fieldstorage_readline(self):
